@@ -194,6 +194,9 @@ class TestOutputPathHandler(object):
         assert output_handler.get_solvent_template_path() == \
             './ligandParams.txt', \
             'Unexpected default solvent parameters path'
+        assert output_handler.get_conformation_library_path() == \
+            './{}.conformation'.format(tag.upper()), \
+            'Unexpected default conformation library path'
 
         # Initialize output handler with an output_path set
         with tempfile.TemporaryDirectory() as tmpdir:
@@ -211,6 +214,10 @@ class TestOutputPathHandler(object):
             assert output_handler.get_solvent_template_path() == \
                 os.path.join(tmpdir, 'output', 'ligandParams.txt'), \
                 'Unexpected default solvent parameters path'
+            assert output_handler.get_conformation_library_path() == \
+                os.path.join(tmpdir, 'output',
+                             '{}.conformation'.format(tag.upper())), \
+                'Unexpected default conformation library path'
 
     def test_datalocal_paths_for_openff(self):
         """It tests the datalocal paths assignment for OpenFF."""
@@ -244,6 +251,11 @@ class TestOutputPathHandler(object):
             create_missing_folders=False) == \
             './DataLocal/OBC/ligandParams.txt', \
             'Unexpected default solvent parameters path'
+        assert output_handler.get_conformation_library_path(
+            create_missing_folders=False) == \
+            './DataLocal/Conformations/' \
+            + '{}.conformation'.format(tag.upper()), \
+            'Unexpected default conformation library path'
 
         with tempfile.TemporaryDirectory() as tmpdir:
             output_handler = OutputPathHandler(
@@ -265,6 +277,11 @@ class TestOutputPathHandler(object):
                 create_missing_folders=False) == \
                 tmpdir + '/output/DataLocal/OBC/ligandParams.txt', \
                 'Unexpected default solvent parameters path'
+            assert output_handler.get_conformation_library_path(
+                create_missing_folders=False) == \
+                tmpdir + '/output/DataLocal/Conformations/' \
+                + '{}.conformation'.format(tag.upper()), \
+                'Unexpected default conformation library path'
 
     def test_datalocal_paths_for_opls(self):
         """It tests the datalocal paths assignment for OPLS2005."""
@@ -299,6 +316,11 @@ class TestOutputPathHandler(object):
             create_missing_folders=False) == \
             './DataLocal/OBC/ligandParams.txt', \
             'Unexpected default solvent parameters path'
+        assert output_handler.get_conformation_library_path(
+            create_missing_folders=False) == \
+            './DataLocal/Conformations/' \
+            + '{}.conformation'.format(tag.upper()), \
+            'Unexpected default conformation library path'
 
         # Initialize output handler with an output_path set
         with tempfile.TemporaryDirectory() as tmpdir:
@@ -321,6 +343,11 @@ class TestOutputPathHandler(object):
                 os.path.join(tmpdir, 'output',
                              'DataLocal/OBC/ligandParams.txt'), \
                 'Unexpected default solvent parameters path'
+            assert output_handler.get_conformation_library_path(
+                create_missing_folders=False) == \
+                os.path.join(tmpdir, 'output/DataLocal/Conformations/' \
+                + '{}.conformation'.format(tag.upper())), \
+                'Unexpected default conformation library path'
 
     def test_datalocal_paths_for_offopls(self):
         """
@@ -360,6 +387,11 @@ class TestOutputPathHandler(object):
             create_missing_folders=False) == \
             './DataLocal/OBC/ligandParams.txt', \
             'Unexpected default solvent parameters path'
+        assert output_handler.get_conformation_library_path(
+            create_missing_folders=False) == \
+            './DataLocal/Conformations/' \
+            + '{}.conformation'.format(tag.upper()), \
+            'Unexpected default conformation library path'
 
         # Initialize output handler with an output_path set
         with tempfile.TemporaryDirectory() as tmpdir:
@@ -382,6 +414,11 @@ class TestOutputPathHandler(object):
                 os.path.join(tmpdir, 'output',
                              'DataLocal/OBC/ligandParams.txt'), \
                 'Unexpected default solvent parameters path'
+            assert output_handler.get_conformation_library_path(
+                create_missing_folders=False) == \
+                os.path.join(tmpdir, 'output/DataLocal/Conformations/' \
+                + '{}.conformation'.format(tag.upper())), \
+                'Unexpected default conformation library path'
 
         # Initialize output handler without output_path
         output_handler = OutputPathHandler(molecule, hybridff,
@@ -405,6 +442,11 @@ class TestOutputPathHandler(object):
             create_missing_folders=False) == \
             './DataLocal/OBC/ligandParams.txt', \
             'Unexpected default solvent parameters path'
+        assert output_handler.get_conformation_library_path(
+            create_missing_folders=False) == \
+            './DataLocal/Conformations/' \
+            + '{}.conformation'.format(tag.upper()), \
+            'Unexpected default conformation library path'
 
         # Initialize output handler with an output_path set
         with tempfile.TemporaryDirectory() as tmpdir:
@@ -427,6 +469,12 @@ class TestOutputPathHandler(object):
                 os.path.join(tmpdir, 'output',
                              'DataLocal/OBC/ligandParams.txt'), \
                 'Unexpected default solvent parameters path'
+            assert output_handler.get_conformation_library_path(
+                create_missing_folders=False) == \
+                os.path.join(tmpdir, 'output',
+                             'DataLocal/Conformations/',
+                             '{}.conformation'.format(tag.upper())), \
+                'Unexpected default conformation library path'
 
     def test_folder_creation(self):
         """
@@ -464,6 +512,11 @@ class TestOutputPathHandler(object):
             path_dir = os.path.dirname(path)
             assert os.path.isdir(path_dir) is False, \
                 'This directory should not exist'
+            path = output_handler.get_conformation_library_path(
+                create_missing_folders=False)
+            path_dir = os.path.dirname(path)
+            assert os.path.isdir(path_dir) is False, \
+                'This directory should not exist'
 
         # Initialize output handler with an output_path set
         with tempfile.TemporaryDirectory() as tmpdir:
@@ -489,11 +542,18 @@ class TestOutputPathHandler(object):
                 path_dir = os.path.dirname(path)
                 assert os.path.isdir(path_dir) is True, \
                     'This directory should exist'
+                path = output_handler.get_conformation_library_path(
+                    create_missing_folders=True)
+                path_dir = os.path.dirname(path)
+                assert os.path.isdir(path_dir) is True, \
+                    'This directory should exist'
+
 
 class TestMAEParser(object):
     """
     It contains all the tests to validate functions that parse MAE files.
     """
+
     def test_parse_charges_from_mae(self):
         """
         It tests the MAE parser for assigning partial charges from an external
@@ -506,78 +566,78 @@ class TestMAEParser(object):
         FORCEFIELD = 'openff_unconstrained-1.2.0.offxml'
 
         PATH_MAE_MAT = get_data_file_path('tests/MAT.mae')
-        PATH_PDB_MAT= get_data_file_path('tests/MAT.pdb')
+        PATH_PDB_MAT = get_data_file_path('tests/MAT.pdb')
         PATH_MAE_BHP = get_data_file_path('ligands/BHP.mae')
         PATH_PDB_BHP = get_data_file_path('ligands/BHP.pdb')
         PATH_MAE_ETL = get_data_file_path('ligands/ethylene.mae')
         PATH_PDB_ETL = get_data_file_path('ligands/ethylene.pdb')
         PATH_PDB_MAL = get_data_file_path('ligands/malonate.pdb')
 
-        CHARGES_REFERENCE_BHP = [[-0.35703,  'O1'],
-                                [-0.59535,  'O2'],
-                                [-0.50292,  'O3'],
-                                [-0.25243,  'C1'],
-                                [0.30438 ,  'C2'],
-                                [0.22092 ,  'C3'],
-                                [0.54336 ,  'C4'],
-                                [-0.20569,  'C5'],
-                                [-0.20192 ,  'C6'],
-                                [0.16631 ,  'C7'],
-                                [0.02422 ,  'C8'],
-                                [-0.09115,  'C9'],
-                                [-0.09904,  'C10'],
-                                [-0.15673,  'C11'],
-                                [-0.13245,  'C12'],
-                                [-0.17806 ,  'C13'],
-                                [-0.12489,  'C14'],
-                                [-0.09307 ,  'C15'],
-                                [-0.08973,  'C16'],
-                                [0.05397 ,  'H1'],
-                                [0.07338 ,  'H2'],
-                                [0.04514 ,  'H3'],
-                                [0.12979 ,  'H4'],
-                                [0.11025 ,  'H5'],
-                                [0.04054 ,  'H6'],
-                                [0.04581 ,  'H7'],
-                                [0.11444 ,  'H8'],
-                                [0.11761 ,  'H9'],
-                                [0.37274 ,  'H10'],
-                                [0.11825 ,  'H11'],
-                                [0.12584 ,  'H12'],
-                                [0.1381 ,  'H13'],
-                                [0.11696 ,  'H14'],
-                                [0.11148 ,  'H15'],
-                                [0.10697 ,  'H16']]
+        CHARGES_REFERENCE_BHP = [[-0.35703, 'O1'],
+                                 [-0.59535, 'O2'],
+                                 [-0.50292, 'O3'],
+                                 [-0.25243, 'C1'],
+                                 [0.30438, 'C2'],
+                                 [0.22092, 'C3'],
+                                 [0.54336, 'C4'],
+                                 [-0.20569, 'C5'],
+                                 [-0.20192, 'C6'],
+                                 [0.16631, 'C7'],
+                                 [0.02422, 'C8'],
+                                 [-0.09115, 'C9'],
+                                 [-0.09904, 'C10'],
+                                 [-0.15673, 'C11'],
+                                 [-0.13245, 'C12'],
+                                 [-0.17806, 'C13'],
+                                 [-0.12489, 'C14'],
+                                 [-0.09307, 'C15'],
+                                 [-0.08973, 'C16'],
+                                 [0.05397, 'H1'],
+                                 [0.07338, 'H2'],
+                                 [0.04514, 'H3'],
+                                 [0.12979, 'H4'],
+                                 [0.11025, 'H5'],
+                                 [0.04054, 'H6'],
+                                 [0.04581, 'H7'],
+                                 [0.11444, 'H8'],
+                                 [0.11761, 'H9'],
+                                 [0.37274, 'H10'],
+                                 [0.11825, 'H11'],
+                                 [0.12584, 'H12'],
+                                 [0.1381, 'H13'],
+                                 [0.11696, 'H14'],
+                                 [0.11148, 'H15'],
+                                 [0.10697, 'H16']]
 
-        CHARGES_REFERENCE_MAT = [[-0.18938 , 'F1' ],
-                                [-0.21715 , 'F2'],
-                                [-0.21234 , 'F3'],
-                                [-0.39736 , 'O1'],
-                                [-0.58890 , 'O2'],
-                                [-1.00825 , 'N1'],
-                                [0.72066  , 'C1'],
-                                [-0.06281 , 'C2'],
-                                [-0.67474 , 'C3'],
-                                [0.10391  , 'C4'],
-                                [0.16293  , 'C5'],
-                                [-0.61076 , 'C6'],
-                                [0.78183  , 'C7'],
-                                [0.27041  , 'C8'],
-                                [-0.48769 , 'C9'],
-                                [0.15704  , 'C10'],
-                                [-0.02646 , 'H1'],
-                                [-0.08394 , 'H2'],
-                                [-0.01308 , 'H3'],
-                                [0.14006  , 'H4'],
-                                [0.12960  , 'H5'],
-                                [0.31245  , 'H6'],
-                                [0.30268  , 'H7'],
-                                [0.17026  , 'H8'],
-                                [0.15782  , 'H9'],
-                                [0.03175  , 'C11'],
-                                [0.03894  , 'H10'],
-                                [0.07509  , 'H11'],
-                                [0.01743  , 'H12']]
+        CHARGES_REFERENCE_MAT = [[-0.18938, 'F1'],
+                                 [-0.21715, 'F2'],
+                                 [-0.21234, 'F3'],
+                                 [-0.39736, 'O1'],
+                                 [-0.58890, 'O2'],
+                                 [-1.00825, 'N1'],
+                                 [0.72066, 'C1'],
+                                 [-0.06281, 'C2'],
+                                 [-0.67474, 'C3'],
+                                 [0.10391, 'C4'],
+                                 [0.16293, 'C5'],
+                                 [-0.61076, 'C6'],
+                                 [0.78183, 'C7'],
+                                 [0.27041, 'C8'],
+                                 [-0.48769, 'C9'],
+                                 [0.15704, 'C10'],
+                                 [-0.02646, 'H1'],
+                                 [-0.08394, 'H2'],
+                                 [-0.01308, 'H3'],
+                                 [0.14006, 'H4'],
+                                 [0.12960, 'H5'],
+                                 [0.31245, 'H6'],
+                                 [0.30268, 'H7'],
+                                 [0.17026, 'H8'],
+                                 [0.15782, 'H9'],
+                                 [0.03175, 'C11'],
+                                 [0.03894, 'H10'],
+                                 [0.07509, 'H11'],
+                                 [0.01743, 'H12']]
 
         # Check up correct charges for malonate
         m = Molecule(PATH_PDB_MAT)
@@ -588,11 +648,11 @@ class TestMAEParser(object):
         for charge, atom_name in zip(parameters['charges'],
                                      parameters['atom_names']):
             assert [charge._value, atom_name.replace('_', '')] in \
-            CHARGES_REFERENCE_MAT, \
-            'Incorrect charge value for {}.'.format(atom_name)
+                CHARGES_REFERENCE_MAT, \
+                'Incorrect charge value for {}.'.format(atom_name)
 
         # Check up correct charges for BHP
-        m = Molecule (PATH_PDB_BHP)
+        m = Molecule(PATH_PDB_BHP)
         openff = OpenForceField(FORCEFIELD)
         parameters = openff.parameterize(m, charge_method='dummy')
         parameters = parse_charges_from_mae(PATH_MAE_BHP, parameters)
@@ -600,8 +660,8 @@ class TestMAEParser(object):
         for charge, atom_name in zip(parameters['charges'],
                                      parameters['atom_names']):
             assert [charge._value, atom_name.replace('_', '')] in \
-            CHARGES_REFERENCE_BHP, \
-            'Incorrect charge value for {}.'.format(atom_name)
+                CHARGES_REFERENCE_BHP, \
+                'Incorrect charge value for {}.'.format(atom_name)
 
         # Error: MAE file without charges information
         m = Molecule(PATH_PDB_ETL)
@@ -618,5 +678,169 @@ class TestMAEParser(object):
             _ = parse_charges_from_mae(PATH_PDB_BHP, parameters)
 
 
+class TestPDBFile(object):
+    """
+    It contains all the tests to validate the PDBFile class.
+    """
+    def test_get_molecule_from_chain(self):
+        """
+        It tests the method that returns a molecule from a selected chain from
+        an input PDB file.
+        """
+        def compare_molecules(mol1, mol2):
+            """
+            It compares that two molecules contain the same PDB atom names and
+            that have been initialized with the same flags.
+
+            Parameters
+            ----------
+            mol1 : a peleffy.topology.Molecule object
+                Molecule.
+            mol2 : a peleffy.topology.Molecule object
+                Reference molecule.
+
+            Returns
+            -------
+            are_equal : bool
+                Wheter the molecules are equal or not.
+            """
+            if mol1._rotamer_resolution == mol2._rotamer_resolution \
+                    and mol1._exclude_terminal_rotamers ==  \
+            mol2._exclude_terminal_rotamers\
+                    and mol1._allow_undefined_stereo == \
+            mol2._allow_undefined_stereo\
+                    and mol1.get_pdb_atom_names() == \
+            mol2.get_pdb_atom_names():
+                return True
+            else:
+                return False
+
+        from peleffy.utils.input import PDBFile
+        from peleffy.utils import get_data_file_path
+        from peleffy.topology import Molecule
+
+        PATH_COMPLEX_PDB = get_data_file_path('complexes/LYS_BNZ.pdb')
+        PATH_LIGAND_PDB = get_data_file_path('ligands/BNZ.pdb')
+
+        # Test method get_molecule_from_chain
+        PDBreader = PDBFile(PATH_COMPLEX_PDB)
+        molecules = PDBreader.get_molecules_from_chain(selected_chain='L')
+        assert len(molecules) == 1, 'Unexpected number of molecules'
+        assert compare_molecules(molecules[0], Molecule(PATH_LIGAND_PDB)) is True
+
+        # Test allow_undefined_stereo flag
+        PDBreader = PDBFile(PATH_COMPLEX_PDB)
+        molecules = \
+            PDBreader.get_molecules_from_chain(selected_chain='L',
+                                               allow_undefined_stereo=True)
+        assert len(molecules) == 1, 'Unexpected number of molecules'
+        assert compare_molecules(molecules[0],
+                                 Molecule(PATH_LIGAND_PDB,
+                                          allow_undefined_stereo=True)) is True
+
+        # Test exclude_terminal_rotamers flag
+        PDBreader = PDBFile(PATH_COMPLEX_PDB)
+        molecules = \
+            PDBreader.get_molecules_from_chain(selected_chain='L',
+                                               exclude_terminal_rotamers=False)
+        assert len(molecules) == 1, 'Unexpected number of molecules'
+        assert compare_molecules(molecules[0], Molecule(
+            PATH_LIGAND_PDB, exclude_terminal_rotamers=False)) is True
+
+        # Test rotamer_resolution flag
+        PDBreader = PDBFile(PATH_COMPLEX_PDB)
+        molecules = PDBreader.get_molecules_from_chain(selected_chain='L',
+                                                       rotamer_resolution=10)
+        assert compare_molecules(molecules[0],
+                                 Molecule(PATH_LIGAND_PDB,
+                                          rotamer_resolution=10)) is True
+
+        # Test core_constraints flag
+        PDBreader = PDBFile(PATH_COMPLEX_PDB)
+        molecules = PDBreader.get_molecules_from_chain(selected_chain='L',
+                                                       core_constraints=[1,2])
+        assert len(molecules) == 1, 'Unexpected number of molecules'
+        assert molecules[0].core_constraints == [1,2]
 
 
+    def test_get_hetero_molecules(self):
+        """
+        It tests the method that returns a list with all the hetero molecules in
+        an input PDB.
+        """
+        def compare_molecules(mol1, mol2):
+            """
+            It compares that two molecules contain the same PDB atom names and
+            that have been initialized with the same flags.
+
+            Parameters
+            ----------
+            mol1 : a peleffy.topology.Molecule object
+                Molecule.
+            mol2 : a peleffy.topology.Molecule object
+                Reference molecule.
+
+            Returns
+            -------
+            are_equal : bool
+                Wheter the molecules are equal or not.
+            """
+            if mol1._rotamer_resolution == mol2._rotamer_resolution \
+                    and mol1._exclude_terminal_rotamers ==  \
+            mol2._exclude_terminal_rotamers\
+                    and mol1._allow_undefined_stereo == \
+            mol2._allow_undefined_stereo\
+                    and mol1.get_pdb_atom_names() == \
+            mol2.get_pdb_atom_names():
+                return True
+            else:
+                return False
+
+        from peleffy.utils.input import PDBFile
+        from peleffy.utils import get_data_file_path
+        from peleffy.topology import Molecule
+
+        PATH_COMPLEX_PDB = get_data_file_path('complexes/LIG1_LIG2.pdb')
+        PATH_LIGAND1 = get_data_file_path('ligands/LIG1.pdb')
+        PATH_LIGAND2 = get_data_file_path('ligands/LIG2.pdb')
+
+        # Test extract multiple molecules
+        PDBreader = PDBFile(PATH_COMPLEX_PDB)
+        molecules = PDBreader.get_hetero_molecules()
+
+        for molecule in molecules:
+            for molecule_ref in [Molecule(PATH_LIGAND1),Molecule(PATH_LIGAND2)]:
+                if molecule._tag == molecule_ref.tag:
+                    assert compare_molecules(molecule, molecule_ref)
+
+        # Test set ligand core constraints
+        PATH_COMPLEX_PDB = get_data_file_path('complexes/complex_test.pdb')
+        molecules = \
+            PDBreader.get_hetero_molecules(allow_undefined_stereo = True,
+                                           ligand_core_constraints = [1,2,3],
+                                           ligand_resname = 'ANL')
+        for molecule in molecules:
+            if molecule._tag == 'ANL':
+                assert molecule.core_constraints == [1,2,3]
+            else:
+                assert molecule.core_constraints == []
+
+
+    def test_raise_errors(self):
+        """
+        It tests all the possible errors when geting a molecule from a chain.
+        """
+        from peleffy.utils.input import PDBFile
+        from peleffy.utils import get_data_file_path
+
+        PATH_COMPLEX_PDB = get_data_file_path('complexes/LYS_BNZ.pdb')
+
+        # The chain selected does not exist in the PDB file
+        with pytest.raises(ValueError):
+            PDBreader = PDBFile(PATH_COMPLEX_PDB)
+            _ = PDBreader.get_molecules_from_chain(selected_chain='F')
+
+        # The chain selected is not an hetero molecule
+        with pytest.raises(ValueError):
+            PDBreader = PDBFile(PATH_COMPLEX_PDB)
+            _ = PDBreader.get_molecules_from_chain(selected_chain='A')
